
LabelPoint::usage = "LabelPoint[f, u, x, du] labels the point f[u] with the 
text u at the end of the principal normal of length x, where f: R -> R ^ 3."

ColouredCurvatures::usage = "ColouredCurvatures[f, l1, l2, x, du] gives the 
graph of the function f: [-1, 1] ->  R ^ 3 at the list l1 coloured according 
to the curvature. There are labels at the points of the list f[l2]."

ColouredTorsions::usage = "ColouredTorsions[f, l1, l2, x, du] gives the graph 
of the function f: [ -1, 1] -> R ^ 3 at the list l1 coloured according to the 
torsion. There are labels at the points of the list f[l2]."

LabelPoint[f_, u_?NumberQ, x_?NumberQ, du_:.01] := 
	Text[u, PrincipalNormal[f, u, x, du][[1, 2]]] 

LabelPoint[f_, l_List, x_?NumberQ, du_:.01] := 
	LabelPoint[f, #, x, du] & /@ l 

ColouredCurvatures[f_, l1_List, l2_List, x_?NumberQ, du_:.01] := Module[
	{m, rgb1, max, min, rgb2, t, colours, lines},
	m = (Drop[l1, 1] + Drop[l1, -1]) / 2;
	rgb1 = 2 / N[Pi] Re @ ArcTan[Curvature[f, m, du]];
	max = Chop @ Max[rgb1];
	min = Chop @ Min[rgb1];
	If[min == max, rgb2 = Table[.5, {t, Length[l1] - 1}],
		rgb2 = (rgb1 - min) / (max - min)];
	colours =  RGBColor[#, 0, 1 - #] & /@ rgb2;
	lines = Line /@ Partition[f /@ l1, 2, 1];
	{
		Thread @ {colours, lines}, 
		{Blue, Point[f[#]] & /@ l2},
		{Blue, LabelPoint[f, l2, x, du]}
	}
] 

ColouredTorsions[f_, l1_List, l2_List, x_?NumberQ, du_:.01] := Module[
	{m, rgb1, max, min, t, rgb2, colours, lines},
	m = (Drop[l1, 1] + Drop[l1, -1]) / 2;
	rgb1 = 2 / N[Pi] Re @ ArcTan[Torsion[f, m]];
	max = Chop @ Max[rgb1];
	min = Chop @ Min[rgb1];
	If[min == max, rgb2 = Table[.5, {t, Length[l1] - 1}], 
		rgb2 = (rgb1 - min) / (max - min)];
	colours =  RGBColor[#, 0, 1 - #] & /@ rgb2;
	lines = Line /@ Partition[f /@ l1, 2, 1];
	{
		Thread @ {colours, lines}, 
		{Blue, Point[f[#]] & /@ l2},
		{Blue, LabelPoint[f, l2, x, du]}
	}
] 

