
OsculatingCircle::usage = "OsculatingCircle[f, u, du] gives the circle passing 
through three points close to f[u] on the curve f: R -> R ^ 3."

OsculatingCircles::usage = "OsculatingCircles[f, l, n, du] gives graphics for 
the curve f: R -> R ^ 3 at n equally spaced points from -1 to 1 with the
circles of curvature at f[l], where l is a list or number." 

OsculatingCircles3D::usage = "OsculatingCircles3D[f, l, n, du] gives graphics 
for the curve f: R -> R ^ 3 at n equally spaced points from -1 to 1 with the 
circles of curvature at f[l], where l is a list or number. This differs from 
OsculatingCircles in that the curve and circles are draw twice: offset and of 
different thicknesses."

AnimateOsculatingCircles::usage = "AnimateOsculatingCircles[f, T, n, du] gives 
T frames of an animation, showing the curve f: R -> R ^ 3 from -1 to 1 
together with its moving circles of curvature (using OsculatingCircles3D)."

AnimateOsculatingCircles3D::usage = 
"AnimateOsculatingCircles3D[m, f, T, n, du] gives T frames of an animation, 
showing the curve f: R -> R ^ 3 from -1 to 1 together with m equally spaced 
moving circles of curvature."

OsculatingCircle[f_, u_?NumberQ, du_:.01] := 
	Circumcircle[{f[u - du], f[u], f[u + du]}] 

OsculatingCircle[f_, l_List, du_:.01] := OsculatingCircle[f, #, du] & /@ l 

OsculatingCircles[f_, l_List, n_:120, du_:.01] := Module[
	{curve, points, circles},
	curve = {Blue, Thickness[.0025], Line[f /@ Range[-1, 1, 2 / (n - 1)]]};
	points = {Blue, Point[f[#]] & /@ l};
	circles = {Red, OsculatingCircle[f, l, du]};
	{curve, points, circles, curve, points} ] 

OsculatingCircles[f_, u_?NumberQ, n_:120, du_:.01] := 
	OsculatingCircles[f, {u}, n, du] 

OsculatingCircles3D[f_, l_List, n_:120, du_:.01] := 
	{
	{Thickness[.02], Blue, #, Thickness[.008], Cyan, 
		Translate[.01 ViewPoint /. Options[Graphics3D]][#]} & @ 
		Line[f /@ Range[-1, 1, 2 / (n - 1)]],
	{Blue, Point[f[#]] & /@ l};
	{Thickness[.02], Yellow, #, Thickness[.008], Red, 
		Translate[.01 ViewPoint /. Options[Graphics3D]][#]} & @ 
		OsculatingCircle[f, l, du]
	}

OsculatingCircles3D[f_, u_?NumberQ, n_:120, du_:.01] := 
	OsculatingCircles3D[f, {u}, n, du] 

AnimateOsculatingCircles[f_, T_?NumberQ, n_:120, du_:.01] := Module[
	{t},
	Table[
		Graphics3D[
			PreDraw3D @ OsculatingCircles[f, t, n, du], 
			PlotRange -> PlotRangeGraphics3D[f]
		], 
		{t, -1, 1 - 2 / (T - 1), 2 / (T - 1)}
	] 
] 

AnimateOsculatingCircles3D[m_, f_, T_?NumberQ, n_:120, du_:.01] := Module[
	{t},
	Table[
		Graphics3D[
			PreDraw3D @ OsculatingCircles3D[
				f, 
				Range[
					2 / m t / T - 1, 
					1 - 2 / m + 2 / m t / T, 
					2 / m
				], 
				n, 
				du
			],
			PlotRange -> PlotRangeGraphics3D[f]
		],
		{t, T}
	] 
]

