
Trihedrons::usage = "Trihedrons[f, l, x, n, du] gives the graphics for the 
curve f: R -> R ^ 3 at n equally spaced points from -1 to 1 with tangents, 
principal normals and binormals at f[l], where l is a list or number." 

AnimateTrihedrons::usage = "AnimateTrihedrons[f, T, x, n, du] gives T frames 
of an animation, showing the curve f: R -> R ^ 3 from -1 to 1 together with 
its moving tangent, principal normal and binormal."

Trihedrons[f_, l_List, x_?NumberQ, n_:120, du_:.01] := Module[
	{curve, tangents, normals, binormals},
	curve = {Blue, Thickness[.0025], Line[f /@ Range[-1, 1, 2 / (n - 1)]]};
	tangents = {Red, Tangent[f, l, x, du]};
	normals = {Blue, PrincipalNormal[f, l, x, du]};
	binormals = {Green, Binormal[f, l, x, du]};
	{curve, tangents, normals, binormals, curve} 
] 

Trihedrons[f_, u_?NumberQ, x_?NumberQ, n_:120, du_:.01] := 
	Trihedrons[f, {u}, x, n, du] 

AnimateTrihedrons[f_, T_?NumberQ, x_?NumberQ, n_:120, du_:.01] := Module[
	{t},
	Table[
		Graphics3D[
			PreDraw3D @ Trihedrons[f, t, x, n, du], 
			PlotRange -> PlotRangeGraphics3D[f]
		], 
		{t, -1, 1 - 2 / (T - 1), 2 / (T - 1)}
	] 
] 

