
GeodesicStep::usage = "GeodesicStep[f, p, d, ds] gives the point r in R ^ 2 
so that f[r] is approximately on the geodesic from f[p] to f[q], where 
f: R ^ 2 -> R ^ 3 and q is a point close to p in the direction d from p."

GeodesicSteps::usage = "GeodesicSteps[f, p, d, ds, b, c] gives a line of 
colour c of b points on the surface given by f: R ^ 2 -> R ^ 3, starting with 
p and q (a point close to p in the direction of d from p). The rest of the 
points are formed by taking GeodesicStep b - 2 times."

GeodesicStep[f_, p:{_, _, _}, d:{_, _, _}, ds_] := Module[
	{q, r, t, n, rule},
	q = p + ds Unit[d];
	r = q + ds (Cos[t] $i + Sin[t] $j);
	rule = Last[FindMinimum[- Angle[f[p] - f[q], f[r] - f[q]], 
		{t, {0, ds}}]];
	r /. rule ] 

GeodesicSteps[f_, p:{_, _, _}, d:{_, _, _}, ds_, b_?NumberQ, c_] := Module[
	{q, l, pp, qq},
	q = p + ds Unit[d];
	l = {p, q};
	Do[
		{pp, qq} = Take[l, -2];
		l = Append[l, GeodesicStep[f, pp, qq - pp, ds]], 
		{b - 2}];
	{Thickness[.0035], c, Line[f /@ l]} ] 

TorusGeodesicSteps[mm_:13, nn_:13] := Draw3D @ 
	{
		OctagonBackdrop[Torus, mm, nn, LightBlue, PaleGreen],
		GeodesicSteps[Torus, .9 $i + .5 $j, $i + $j, .01, 600, Red] 
	}

GeodesicSpray[f_, p_, l_, ds_, b_, c_] := 
	GeodesicSteps[f, p, #, ds, b, c] & /@ l 

TopEllipsoidGeodesicSpray[p_, mm_:12, nn_:12] := Draw3D @ 
	GeodesicSpray[TopEllipsoid, p, RegularPolygon[8], .01, 3, Red] 

TopEllipsoidGeodesicSprays[n_:12, b_:12, ds_:.02, mm_:12, nn_:12] := Module[
	{l = RegularPolygon[n]},
	Draw3D @ 
	{
		GridBackdrop[Ellipsoid, mm, nn, Yellow],
		GeodesicSpray[TopEllipsoid, .5 $i + .5 $j, l, ds, b, Red],
		GeodesicSpray[TopEllipsoid, $i + .5 $j, l, ds, b, Green],
		GeodesicSpray[TopEllipsoid, $i, l, ds, b, Blue]
	}
] 

