
OsculatingSphere::usage = "OsculatingSphere[f, u, du] gives the sphere through 
three points near to f[u] on the curve f: R -> R ^ 3."

OsculatingSpheres::usage = "OsculatingSpheres[f, l, n, du] gives graphics for 
the curve f: R -> R ^ 3 at n equally spaced points from -1 to 1 with 
osculating spheres and circles at f[l], where l is a list or number." 

AnimateOsculatingSpheres::usage = "AnimateOsculatingSpheres[f, l, n, du] gives 
T frames of an animation, showing the curve f: R -> R ^ 3 from -1 to 1 
together with its moving circles and spheres of curvature."

OsculatingSphere[f_, u_?NumberQ, du_:.01] := Sphere[
	CentreOfCurvature[f, u, du] , RadiusOfCurvature[f, u, du]
] /; Coplanar @ {f[u - du], f[u], f[u + .5 du], f[u + du]} 

OsculatingSphere[f_, u_?NumberQ, du_:.01] := 
	Circumsphere[{f[u - du], f[u], f[u + .5 du], f[u + du]}] 

OsculatingSphere[f_, l_List, du_:.01] := OsculatingSphere[f, #, du] & /@ l 

OsculatingSpheres[f_, l_List, n_:120, du_:.01] := Module[
	{curve, points, circles, spheres},
	curve = {Blue, Thickness[.0025], Line[f /@ Range[-1, 1, 2 / (n - 1)]]};
	points = {Blue, Point[f[#]] & /@ l};
	circles = {Red, OsculatingCircle[f, l, du]};
	spheres = OsculatingSphere[f, l, du];
	{curve, points, spheres, circles} ] 

OsculatingSpheres[f_, u_?NumberQ, n_:120, du_:.01] := 
	OsculatingSpheres[f, {u}, n, du] 

AnimateOsculatingSpheres[f_, T_?NumberQ, n_:120, du_:.01] := Module[
	{t},
	Table[
		Graphics3D[
			PreDraw3D @ OsculatingSpheres[f, t, n, du],
			PlotRange -> PlotRangeGraphics3D[f],
			Lighting -> True
		], 
		{t, -1, 1 - 2 / (T - 1), 2 / (T - 1)}
	] 
] 

